<?php
/*
 * Description of SFFFormTemplateField
 *
 * This class is distinct from SFTemplateField in that it represents a
 * template field defined in a form - it contains a SFTemplateField object
 * within it (the $template_field variable), along with the other properties
 * for that field that are set within the form
 *
 * @author Rens te Kloese
 */
class SFFFormTemplateField extends SFTemplateField {
    var $num;
    var $template_field;
    var $input_type;
    var $is_mandatory;
    var $is_hidden;
    var $is_restricted;
    var $possible_values;
    // the following fields are not set by the form-creation page
    // (though they could be)
    var $is_uploadable;
    var $field_args;
    var $autocomplete_category;
    var $no_autocomplete;
    var $part_of_multiple;
    // somewhat of a hack - these two fields are for a field in a specific
    // representation of a form, not the form definition; ideally these
    // should be contained in a third 'field' class, called something like
    // SFFormInstanceField, that holds these fields plus an instance of
    // SFFormTemplateField. Too much work?
    var $input_name;
    var $is_disabled;



    // Functions are copied and changed from parent class:
    // new interface field 'label' added to variant
    static function createFromDefinition($field_name, $input_name, $label, $is_mandatory, $is_hidden, $is_uploadable, $possible_values, $is_disabled, $is_list, $input_type, $field_args, $all_fields, $strict_parsing) {
        // see if this field matches one of the fields defined for this template -
        // if it is, use all available information about that field; if it's not,
        // either include it in the form or not, depending on whether the
        // template has a 'strict' setting in the form definition
        $the_field = null;
        foreach ($all_fields as $cur_field) {
            if ($field_name == $cur_field->field_name) {
                $the_field = $cur_field;
                break;
            }
        }

        if ($the_field == null) {
            if ($strict_parsing) {
                $dummy_ftf = new SFFFormTemplateField();
                $dummy_ftf->template_field = new SFTemplateField();
                $dummy_ftf->is_list = false;
                $dummy_ftf->field_args = array();
                return $dummy_ftf;
            }
            $the_field = new SFTemplateField();
        }

        // create an SFFormTemplateField object, containing this field as well
        // as settings from the form definition file
        $f = new SFFFormTemplateField();
        $f->template_field = $the_field;
        
        //added:
        $f->template_field->label = $input_name;
        $f->template_field->input_type = $input_type;
	    $f->template_field->field_name = $field_name;
        $f->template_field->possible_values = $possible_values;
        $f->template_field->is_list = $is_list;

        $f->is_mandatory = $is_mandatory;
        $f->is_hidden = $is_hidden;
        $f->is_uploadable = $is_uploadable;
        $f->possible_values = $possible_values;
        $f->input_type = $input_type;
        $f->field_args = $field_args;
        $f->input_name = $input_name;
        $f->is_disabled = $is_disabled;
        $f->is_list = $is_list;
        return $f;
    }

function creationHTML($template_num) {
        global $wgOut;
        
        $field_form_text = $template_num . "_" . $this->num;
        $template_field = $this->template_field;
        $text = '<h3>' . wfMsg('sf_createform_field') . " '" . $template_field->field_name . "'</h3>\n";
        $prop_link_text = SFUtils::linkText(SMW_NS_PROPERTY, $template_field->semantic_field);
        // TODO - remove this probably-unnecessary check?
        if ($template_field->semantic_field == "") {
            // print nothing if there's no semantic field
        } elseif ($template_field->field_type == "") {
            $text .= '<p>' . wfMsg('sf_createform_fieldpropunknowntype', $prop_link_text) . "</p>\n";
        } elseif ($template_field->is_list) {
            $text .= '<p>' . wfMsg('sf_createform_fieldproplist', $prop_link_text,
                SFUtils::linkText(SMW_NS_TYPE, $template_field->field_type)) . "</p>\n";
        } else {
            $text .= '<p>' . wfMsg('sf_createform_fieldprop', $prop_link_text,
                SFUtils::linkText(SMW_NS_TYPE, $template_field->field_type)) . "</p>\n";
        }
        // if it's not a semantic field - don't add any text
        $form_label_text = wfMsg('sf_createform_formlabel');


        $field_label = $template_field->label;
        $input_type_text = wfMsg('sf_createform_inputtype');
        $text .=<<<END
    <p>$form_label_text <input type="text" name="label_$field_form_text" size=20 value="$field_label" />
    &nbsp; $input_type_text

END;
        $possible_input_types = $this->possibleInputTypes($template_field->field_type, $template_field->is_list);
        if (count($possible_input_types) > 1) {
            $text .= $this->inputTypeDropdownHTML("input_type_$field_form_text", $possible_input_types, $template_field->input_type);
        } else {
            $text .= $possible_input_types[0];
        }
        $mandatory_checked_str = ($this->is_mandatory) ? "checked" : "";
        $mandatory_text = wfMsg('sf_createform_mandatory');
        $hidden_checked_str = ($this->is_hidden) ? "checked" : "";
        $hidden_text = wfMsg('sf_createform_hidden');
        $restricted_checked_str = ($this->is_restricted) ? "checked" : "";
        $restricted_text = wfMsg('sf_createform_restricted');
        $text .=<<<END
    </p>
    <p>
    <input type="checkbox" name="mandatory_$field_form_text" value="mandatory" $mandatory_checked_str /> $mandatory_text
    <input type="checkbox" name="hidden_$field_form_text" value="hidden" $hidden_checked_str /> $hidden_text
    <input type="checkbox" name="restricted_$field_form_text" value="restricted" $restricted_checked_str /> $restricted_text</p>
    <hr>

END;
        return $text;
    }

    // for now, HTML of an individual field depends on whether or not it's
    // part of multiple-instance template; this may change if handling of
    // such templates in form definitions gets more sophisticated
    function createMarkup($part_of_multiple, $is_last_field_in_template) {
        $text = "";
        if ($this->template_field->label != "") {
            if ($part_of_multiple) {
                $text .= "'''" . $this->template_field->label .  ":''' ";
            } else {
                $text .= "! " . $this->template_field->label . ":\n";
            }
        }
        if (! $part_of_multiple) { $text .= "| "; }
        $text .= "{{{field|" . $this->template_field->field_name;
        if ($this->template_field->input_type != '') {
            $text .= "|input type=" . $this->template_field->input_type;
        }
        if ($this->is_mandatory) {
            $text .= "|mandatory";
        } elseif ($this->is_hidden) {
            $text .= "|hidden";
        } elseif ($this->is_restricted) {
            $text .= "|restricted";
        }
        $text .= "}}}\n";
        if ($part_of_multiple) {
            $text .= "\n";
        } elseif (! $is_last_field_in_template) {
            $text .= "|-\n";
        }
        return $text;
    }
}
