<?php

/**
 * Utility class for 'edit with form' tab
 * Partly borrowed from SF_FormEditTab.php
 *
 * @author Yaron Koren (SF)
 * @author Henri Gerrits
 */
class SFFFormEditTab {

    /**
     * Adds an "action" (tab) to edit the current template/form/property with
     * a special page,
     * @param array $contentActions an array of already initialized tab info
     * @return boolean always returns true
     */
    static function displayTab(&$contentActions) {
        $currentTitle = SFFGlobals::getTitle();
        if (isset($currentTitle)) {
            $namespace = $currentTitle->getNamespace();
            if ($namespace == NS_TEMPLATE || $namespace == SF_NS_FORM || $namespace == SMW_NS_PROPERTY) {
                // @todo this will create a tab for ALL templates, forms and properties
                if ($namespace == NS_TEMPLATE) {
                    $specialPage = 'SFF_EditTemplate';
                } else if ($namespace == SF_NS_FORM) {
                    $specialPage = 'SFF_EditForm';
                } else if ($namespace == SMW_NS_PROPERTY) {
                    $specialPage = 'SFF_EditProperty';
                }
                $href = Title::makeTitle(NS_SPECIAL, $specialPage)->getFullURL('name=' . urlencode($currentTitle->getText()));
                self::addEditWithFormTab($currentTitle, $contentActions, $href);
            } else if ($namespace == NS_MAIN) {
                $regex = '/\|\s*SFForm\s*=\s*((?U)[^\|\}]+)\s*[\|\}]/mi';
                $article = new Article($currentTitle);
                $articleText = $article->getContent();
                if (preg_match($regex, $articleText, $matches)) {
                    $specialPage = 'SFF_EditData';
                    $href = Title::makeTitle(NS_SPECIAL, $specialPage)->getFullURL('form=' . urlencode($matches[1]) . '&target=' . urlencode($currentTitle->getText()));
                    self::addEditWithFormTab($currentTitle, $contentActions, $href);
                } else {
                    $formNames = SFLinkUtils::getFormsForArticle(SFFGlobals::getUser()->getSkin());
                    if (count($formNames) > 0) {
                        $specialPage = 'SFF_EditData';
                        $href = Title::makeTitle(NS_SPECIAL, $specialPage)->getFullURL('form=' . urlencode($formNames[0]) . '&target=' . urlencode($currentTitle->getText()));
                        self::addEditWithFormTab($currentTitle, $contentActions, $href);
                    }
                }
            }
        }
        return true; // always return true, in order not to stop MW's hook processing!
    }

    // implementation borrowed from SF_FormEditTab.php, $content_action will be modified, so pass by reference
    /**
     *
     * @global boolean $sfgRenameEditTabs use ('edit with form' and 'edit') if true, or ('edit' and 'edit source') if false. See SF_Settings.php
     * @param Title $title the title of the article that will display the tab
     * @param array $content_actions an array of already initialized tab info
     * @param string $specialPage the simple name of the special page (excluding namespace)
     */
    private static function addEditWithFormTab($title, &$contentActions, $href) {
        $request = SFFGlobals::getRequest();
        $user = SFFGlobals::getUser();

        global $sfgRenameEditTabs;

        wfLoadExtensionMessages('SemanticForms'); // SFF can use SF's messages

        $userCanEdit = $user->isAllowed('edit') && $title->userCanEdit();
        // create the form edit tab, and apply whatever
        // changes are specified by the edit-tab global
        // variables
        if ($sfgRenameEditTabs) {
            $formEditTabText = $userCanEdit ? wfMsg('edit') : wfMsg('sf_viewform');
            if (array_key_exists('edit', $contentActions)) {
                $contentActions['edit']['text'] = $userCanEdit ? wfMsg('sf_editsource') : wfMsg('viewsource');
            }
        } else {
            $formEditTabText = $userCanEdit ? wfMsg('sf_formedit') : wfMsg('sf_viewform');
        }

        $className = ($request->getVal('action') == 'formedit') ? 'selected' : '';
        $formEditTab = array(
            'class' => $className,
            'text' => $formEditTabText,
            'href' => $href
        );

        // find the location of the 'edit' tab, and add
        // 'edit with form' right before it.
        // this is a "key-safe" splice - it preserves
        // both the keys and the values of the array,
        // by editing them separately and then
        // rebuilding the array.
        // based on the example at
        // http://us2.php.net/manual/en/function.array-splice.php#31234
        $tabKeys = array_keys($contentActions);
        $tabValues = array_values($contentActions);
        $editTabLocation = array_search('edit', $tabKeys);
        // if there's no 'edit' tab, look for the
        // 'view source' tab instead
        if ($editTabLocation == NULL)
            $editTabLocation = array_search('viewsource', $tabKeys);
        // this should rarely happen, but if there was
        // no edit *or* view source tab, set the
        // location index to -1, so the tab shows up
        // near the end
        if ($editTabLocation == NULL)
            $editTabLocation = -1;
        array_splice($tabKeys, $editTabLocation, 0, 'form_edit');
        array_splice($tabValues, $editTabLocation, 0, array($formEditTab));
        $contentActions = array();
        for ($i = 0; $i < count($tabKeys); $i++)
            $contentActions[$tabKeys[$i]] = $tabValues[$i];

        if (! $user->isAllowed('viewedittab')) {
        // the tab can have either of those two actions
            unset($contentActions['edit']);
            unset($contentActions['viewsource']);
        }

    }

}
