<?php

/**
 * Global functions and constants for SFF.
 *
 * @author Rens te Kloese
 * @author Henri Gerrits
 * @author Mark Rotteveel
 */

if ( !defined( 'MEDIAWIKI' ) ) die();

define('SFF_VERSION','0.5.0');

$wgExtensionCredits['specialpage'][]= array(
    'name' => 'SFF Extension',
    'version' => SFF_VERSION,
    'author' => 'Rens te Kloese, Henri Gerrits, Mark Rotteveel',
//    'url' => 'http://localhost/wikiwin/Extension:SFF_Extension',
    'description' => 'Extensions for the Open Universiteit NL',
    'descriptionmsg'  => 'sff_ext_description',
);

// set language dependent
$wgExtensionMessagesFiles['SFF_extension'] = $sffgIP . '/languages/SFF_Messages.php';

$wgExtensionFunctions[] = 'sffSetupExtension';

// load accessor class for global MediaWiki variables
$wgAutoloadClasses['SFFGlobals'] = $sffgIP . '/includes/SFF_Globals.php';

// create hook to add 'edit with form' tab before property page rendering
$wgAutoloadClasses['SFFFormEditTab'] = $sffgIP . '/includes/SFF_FormEditTab.php';
$wgHooks['SkinTemplateContentActions'][] = 'SFFFormEditTab::displayTab';

// register all special pages and other classes

//Edit Forms:
$wgAutoloadClasses['SFFFormTemplateField'] = $sffgIP . '/includes/SFFFormTemplateField.php';
$wgAutoloadClasses['SFFTemplateInForm'] = $sffgIP . '/includes/SFFTemplateInForm.php';
$wgSpecialPages['SFF_EditForm'] = 'SFFEditForm';
$wgAutoloadClasses['SFFEditForm'] = $sffgIP . '/specials/SFF_EditForm.php';
$wgSpecialPageGroups['SFF_EditForm'] = 'sff_group';
$wgAutoloadClasses['SFFFormParserFunctions'] = $sffgIP . '/parserfunctions/SFF_FormParserFunctions.php';

//Edit Template:
$wgSpecialPages['SFF_EditTemplate'] = 'SFFEditTemplate';
$wgAutoloadClasses['SFFEditTemplate'] = $sffgIP . '/specials/SFF_TemplateSpecialPages.php';
$wgSpecialPageGroups['SFF_EditTemplate'] = 'sff_group';

//Create Template:
$wgSpecialPages['SFF_CreateTemplate'] = 'SFFCreateTemplate';
$wgAutoloadClasses['SFFCreateTemplate'] = $sffgIP . '/specials/SFF_TemplateSpecialPages.php';
$wgSpecialPageGroups['SFF_CreateTemplate'] = 'sff_group';

// load new template class
$wgAutoloadClasses['SFFTemplate'] = $sffgIP . '/includes/SFF_Template.php';

$wgAutoloadClasses['SFFTemplateField'] = $sffgIP . '/includes/SFF_TemplateField.php';

$wgAutoloadClasses['SFFTemplateParserFunctions'] = $sffgIP . '/parserfunctions/SFF_TemplateParserFunctions.php';

//Edit Property:
$wgHooks['smwInitProperties'][] = 'sffInitPredefinedProperties';
$wgSpecialPages['SFF_EditProperty'] = 'SFFEditProperty';
$wgAutoloadClasses['SFFEditProperty'] = $sffgIP . '/specials/SFF_PropertySpecialPages.php';
$wgSpecialPageGroups['SFF_EditProperty'] = 'sff_group';

//Create Property:
$wgSpecialPages['SFF_CreateProperty'] = 'SFFCreateProperty';
$wgAutoloadClasses['SFFCreateProperty'] = $sffgIP . '/specials/SFF_PropertySpecialPages.php';
$wgSpecialPageGroups['SFF_CreateProperty'] = 'sff_group';

// load new property class
$wgAutoloadClasses['SFFProperty'] = $sffgIP . '/includes/SFF_Property.php';

$wgAutoloadClasses['SFFPropertyParserFunctions'] = $sffgIP . '/parserfunctions/SFF_PropertyParserFunctions.php';

//AddData:
$wgSpecialPages['SFF_AddData'] = 'SFFAddData';
$wgAutoloadClasses['SFFAddData'] = $sffgIP . '/specials/SFF_AddData.php';
$wgSpecialPageGroups['SFF_AddData'] = 'sff_group';

//EditData:
$wgSpecialPages['SFF_EditData'] = 'SFFEditData';
$wgAutoloadClasses['SFFEditData'] = $sffgIP . '/specials/SFF_EditData.php';
$wgSpecialPageGroups['SFF_EditData'] = 'sff_group';

//AddPage:
$wgSpecialPages['SFF_AddPage'] = 'SFFAddPage';
$wgAutoloadClasses['SFFAddPage'] = $sffgIP . '/specials/SFF_AddPage.php';
$wgSpecialPageGroups['SFF_AddPage'] = 'sff_group';

$wgAutoloadClasses['SFFFormPrinter'] = $sffgIP . '/includes/SFF_FormPrinter.php';
$wgAutoloadClasses['SFFForm'] = $sffgIP . '/includes/SFF_Form.php';

//SFF Utilities
$wgAutoloadClasses['SFFUtils'] = $sffgIP . '/includes/SFF_Utils.php';

//SFF Articles
$wgAutoloadClasses['SFFArticleFunctions'] = $sffgIP . '/includes/SFF_ArticleFunctions.php';

// Define setup functions for parser functions
$wgExtensionFunctions[] = 'SFFPropertyParserFunctions::propertyParserFunctions_Setup';
$wgExtensionFunctions[] = 'SFFTemplateParserFunctions::templateParserFunctions_Setup';
$wgExtensionFunctions[] = 'SFFFormParserFunctions::formParserFunctions_Setup';
// Add hooks to initialise the magic words
$wgHooks['LanguageGetMagic'][] = 'SFFPropertyParserFunctions::propertyParserFunctions_Magic';
$wgHooks['LanguageGetMagic'][] = 'SFFTemplateParserFunctions::templateParserFunctions_Magic';
$wgHooks['LanguageGetMagic'][] = 'SFFFormParserFunctions::formParserFunctions_Magic';
// Hook removed while inverseOf and isSymetric functions are not dosplayed properly
//On test this functions, activate next line:
//$wgHooks['ParserAfterTidy'][] = 'SFFFormParserFunctions::SFFonParserAfterTidy';

/**
 *  Do the actual intialization of the extension. This is just a delayed init that makes sure
 *  MediaWiki is set up properly before we add our stuff.
 */
function sffSetupExtension() {
    // this global variable is needed so that other extensions (such
    // as Semantic Google Maps) can hook into it to add their own input
    // types
    global $sffFormPrinter;
    $sffFormPrinter = new StubObject( 'sffFormPrinter', 'SFFFormPrinter' );
}

function sffInitPredefinedProperties() {
    SMWPropertyValue::registerProperty('___SFFRAN', '_wpg', 'Has range', true);
    SMWPropertyValue::registerProperty('___SFFDOM', '_wpg', 'Has domain', true);
    SMWPropertyValue::registerProperty('___SFFINV', '_wpg', 'Is inverse of', true);
    SMWPropertyValue::registerProperty('___SFFSYM', '_boo', 'Is symmetric', true);
    SMWPropertyValue::registerProperty('___SFFLAB', '_str', 'Has label', true);
    SMWPropertyValue::registerProperty('___SFFCOM', '_str', 'Has comment', true);
    return true;
}