<?php

if (!defined('MEDIAWIKI')) die();

/*
 * Provides centralized access to global MediaWiki variables.
 * Currently implemented as static wrapper functions. Still is a global singleton, so not much
 * better...
 * @author Henri Gerrits
 */
class SFFGlobals {

    /**
     * The Language object used by Semantic Forms for internationalization.
     * @global Language $sfgContLang the Semantic Forms Language object
     * @return Language the Semantic Forms Language object
     */
    static function getSFContLang() {
        global $sfgContLang;
        return $sfgContLang;
    }

    /**
     * The Language object used by Semantic MediaWiki for internationalization.
     * @global Language $smwgContLang the Semantic MediaWiki Language object
     * @return Language the Semantic MediaWiki Language object
     */
    static function getSMWContLang() {
        global $smwgContLang;
        return $smwgContLang;
    }

    /**
     * The Language object used by MediaWiki for internationalization.
     * @global Language $wgContLang the MediaWiki Language object
     * @return Language the MediaWiki Language object
     */
    static function getMWContLang() {
        global $wgContLang;
        return $wgContLang;
    }

    /**
     * Returns the symbol used to separate lists.
     * @global String $sfgListSeparator the list separator symbol
     * @return String the list separator symbol
     */
    static function getListSeparator() {
        global $sfgListSeparator;
        return $sfgListSeparator;
    }

    /**
     * Returns the parser object, which is responsible for parsing the tags and
     * wikitext contained in wiki pages.
     * @global Parser $wgParser the parser object
     * @return Parser the parser object
     */
    static function getParser() {
        global $wgParser;
        return $wgParser;
    }

    /**
     * Returns the outputPage object, which is used to constructing the output page.
     * @global OutputPage $wgOut the outputPage object
     * @return OutputPage the output object
     */
    static function getOutputPage() {
        global $wgOut;
        return $wgOut;
    }

    /**
     * Returns the current web request.
     * @global WebRequest $wgRequest the current web request
     * @return WebRequest the current web request. 
     */
    static function getRequest() {
        global $wgRequest;
        return $wgRequest;
    }

    /**
     * Returns the script path
     * @global String $sfgScriptPath the script path
     * @return String the script path
     */
    static function getScriptPath() {
        global $sfgScriptPath;
        return $sfgScriptPath;
    }

    /**
     * Returns the current title (MediaWiki article).
     * @global Title $wgTitle the current title
     * @return Title the current title.
     */
    static function getTitle() {
        global $wgTitle;
        return $wgTitle;
    }

    /**
     * Returns the current user.
     * @global User $wgUser the current user
     * @return User the current user.
     */
    static function getUser() {
        global $wgUser;
        return $wgUser;
    }

}

?>
