<?php
/**
 * Description of SFF_TemplateParserFunctions
 * Following the example of http://www.mediawiki.org/wiki/Manual:Parser_functions
 *
 * @author Henri
 */
class SFFTemplateParserFunctions {

    /**
     * Sets up the parser functions. Sets a function hook for the 'sffeditlink' and 'sffheadrow'
     * magic words.
     */
    static function templateParserFunctions_Setup() {
        $parser = SFFGlobals::getParser();
        // Set hooks associating the 'sffeditlink' and 'sffheadrow' magic words with our functions
        $parser->setFunctionHook('sff_editlink', 'SFFTemplateParserFunctions::editlinkFunction_Render');
        $parser->setFunctionHook('sff_headrow', 'SFFTemplateParserFunctions::headrowFunction_Render');
    }

    /**
     * Registers the 'sffeditlink' and 'sffheadrow' magic words with the MediaWiki server.
     * @param array $magicWords the array of magic words to which it will be added
     * @param String $langCode the language code
     * @return Boolean always returns true
     */
    static function templateParserFunctions_Magic(&$magicWords, $langCode) {
        // Add the magic words
        // The first array element is case sensitive, in this case it is not case sensitive
        // All remaining elements are synonyms for our parser function
        $magicWords['sff_editlink'] = array(0, 'sffeditlink');
        $magicWords['sff_headrow'] = array(0, 'sffheadrow');
        // unless we return true, other parser functions extensions won't get loaded.
        return true;
    }

    /**
     * Expands the 'sffeditlink' function call to a linked
     * @param array anonymous array containing the parser object at [0] and the arguments to the
     * function call in subsequent positions (wikitext with templates expanded)
     * @return String a (HTML) link to an edit
     */
    static function editlinkFunction_Render(&$parser, $inFormName = '', $inPageName = '', $inLinkString = '') {
		$editDataPage = SpecialPage::getPage('SFF_EditData');
		$linkUrl = $editDataPage->getTitle()->getLocalURL() . "/$inFormName/$inPageName";
		$linkUrl = str_replace(' ', '_', $linkUrl);
		$htmlString = "<a href=\"$linkUrl\">$inLinkString</a>";
		return $parser->insertStripItem($htmlString, $parser->mStripState);
    }

    /**
     * Expands the 'sffheadrow' function call to a linked
     * @param array anonymous array containing the parser object at [0] and the arguments to the
     * function call in subsequent positions (wikitext with templates expanded)
     * @return String a (HTML) link to an edit
     */
    static function headrowFunction_Render(&$parser, $inLabel) {
        $string = "<span></span>\n|-\n! $inLabel\n|";
        return $string;
    }

}
?>
