<?php
/*
 * Show the list with available forms
 * click one and edit it
 * OR add a new form.
 */

/**
 * Description of SFF_EditForm
 *
 * @author Rens te Kloese
 */

if (!defined('MEDIAWIKI')) die();

class SFFEditForm extends SpecialPage {


    /**
     * Constructor
     */
    function SFFEditForm() {
        SpecialPage::SpecialPage('SFF_EditForm');
        wfLoadExtensionMessages('SemanticForms');
        wfLoadExtensionMessages('SFF_extension');
    }

    function execute($query) {
        global $wgRequest;

        $this->setHeaders();
        # read url parameter form
        $url_form_name = $wgRequest->getVal('name');
        if (doSpecialEditForm($url_form_name) == '1') {
            // not by 'save' or 'preview'
            list( $limit, $offset ) = wfCheckLimits();
            $rep = new SFF_MaintainForm();
            $rep->doQuery( $offset, $limit );
        }
    }

}

class SFF_MaintainForm extends QueryPage {

    function getPageFooter() {
    }

    function getName() { return( 'SFF EditForm' ); }
    function isExpensive() { return true; }
    function isSyndicated() { return false; }
    function sortDescending() { return false; }
    function getOrder() { return false; }

    function getSQL() {
        $dbr = wfGetDB( DB_SLAVE );
        list( $page ) = $dbr->tableNamesN( 'page' );
        $sql = "SELECT page_title as title
            FROM $page
            WHERE page_namespace = 106
            ORDER BY page_title";
        return $sql;
    }

    function formatResult( $skin, $result ) {
        $spedfrm = SpecialPage::getPage('SFF_EditForm');
        $title = $spedfrm->getTitle();
        $pageLink = $skin->makeKnownLinkObj( $title, str_replace('_', ' ', $result->title), 'name=' . $result->title );
        return wfSpecialList( $pageLink, '' );
    }

    function getPageHeader() {
        wfLoadExtensionMessages('SemanticForms');
        wfLoadExtensionMessages('SFF_extension');
        $header = wfMsgExt( 'sff_editform_extended', array( 'parse' ) );
        return $header;
    }
}

# The function is composed with copied parts from the Class SF_CreateForm
# and is modified for the special page SFF_EditForm
function doSpecialEditForm($url_form_name) {
    global $wgOut, $wgRequest, $wgUser, $sfgScriptPath, $wgContLang;
    $db = wfGetDB( DB_SLAVE );

    # read SFF messages
    wfLoadExtensionMessages('SFF_extension');
    $edit_form_text = wfMsg('sff_editform_add');

    # set message pool to SF
    wfLoadExtensionMessages('SemanticForms');

    # init values
    $form_exists = false;

    # get the names of all templates on this site
    $all_templates = array();
    $sql = "SELECT page_title FROM {$db->tableName( 'page' )} WHERE page_namespace=" . NS_TEMPLATE . " AND page_is_redirect = 0 ORDER BY page_title";
    $res = $db->query( $sql );
    if ($db->numRows( $res ) > 0) {
        while ($row = $db->fetchRow($res)) {
            $template_name = str_replace('_', ' ', $row[0]);
            $all_templates[] = $template_name;
        }
    }

    $form_templates = array();
    $i = 1;
    $deleted_template_loc = null;

    # handle inputs
    if ($url_form_name != '' && ! $wgRequest->wasPosted() ) {
        $form_name = str_replace('_', ' ', $url_form_name);
        // Force to fill templates and fields as in blob
        $wgRequest->setVal( 'maintain_form', '1' );
    }
    else {
        $form_name = $wgRequest->getVal('form_name');
    }
    foreach ($wgRequest->getValues() as $var => $val) {
        # ignore variables that are not of the right form
        if (strpos($var, "_") != false) {
            # get the template declarations and work from there
            list ($action, $id) = explode("_", $var, 2);
            if ($action == "template") {
                # if the button was pressed to remove this
                # template, just don't add it to the array
                if ($wgRequest->getVal("del_$id") != null) {
                    $deleted_template_loc = $id;
                } else {
                    $form_template = SFFTemplateInForm::create($val,
                        $wgRequest->getVal("label_$id"),
                        $wgRequest->getVal("allow_multiple_$id"));
                    $form_templates[] = $form_template;
                }
            }
        }
    }


    # on template button
    if ($wgRequest->getVal('add_template') != null) {
        $form_template = SFFTemplateInForm::create($wgRequest->getVal('new_template'), "", false);
        $new_template_loc = $wgRequest->getVal('before_template');
        if ($new_template_loc === null) { $new_template_loc = 0; }
        # hack - array_splice() doesn't work for objects, so we have to
        # first insert a stub element into the array, then replace that
        # with the actual object
        array_splice($form_templates, $new_template_loc, 0, "stub");
        $form_templates[$new_template_loc] = $form_template;
    } else {
        $new_template_loc = null;
    }

    # now cycle through the templates and fields, modifying each one
    # per the query variables
    foreach ($form_templates as $i => $ft) {
        foreach ($ft->fields as $j => $field) {
            // handle the change in indexing if a new template was
            // inserted before the end, or one was deleted
            $old_i = $i;
            if ($new_template_loc != null) {
                if ($i > $new_template_loc) {
                    $old_i = $i - 1;
                } elseif ($i == $new_template_loc) {
                    // it's the new template; it shouldn't
                    // get any query string data
                    $old_i = -1;
                }
            } elseif ($deleted_template_loc != null) {
                if ($i >= $deleted_template_loc) {
                    $old_i = $i + 1;
                }
            }
            $new_label = $wgRequest->getVal("label_" . $old_i . "_" . $j);
            if ($new_label)
            $field->template_field->label = $new_label;
            $input_type = $wgRequest->getVal("input_type_" . $old_i . "_" . $j);
            $field->template_field->input_type = $input_type;
            if ($wgRequest->getVal("hidden_" . $old_i . "_" . $j) == "hidden") {
                $field->is_hidden = true;
            }
            if ($wgRequest->getVal("restricted_" . $old_i . "_" . $j) == "restricted") {
                $field->is_restricted = true;
            }
            if ($wgRequest->getVal("mandatory_" . $old_i . "_" . $j) == "mandatory") {
                $field->is_mandatory = true;
            }
        }
    }

    # check the 'add or edit' button pressed,
    # than determine if form exists or not
    if ($wgRequest->getVal('maintain_form') != null) {
        if ($form_name == "") {
            $form_name_error_str = wfMsg('sf_blank_error');
            $new_template_loc = null;
        } else {
            $form_templates = doEditExistingForm($form_name, $form_templates, $db);
            if (count($form_templates) == 0) {
                $form_exists=false;
            } else {
                $form_exists=true;
            }
        }
    }

    $form = SFFForm::create($form_name, $form_templates);
    # if submit button was pressed, create the form definitions file, then redirect
    $save_page = $wgRequest->getCheck('wpSave');
    $preview_page = $wgRequest->getCheck('wpPreview');
    if ($save_page || $preview_page) {
        # validate form name
        if ($form->form_name == "") {
            $form_name_error_str = wfMsg('sf_blank_error');
        } else {
            $form_name_error_str = '';
            # redirect to wiki interface
            $wgOut->setArticleBodyOnly(true);
            $title = Title::newFromText($form->form_name, SF_NS_FORM);
            $full_text = str_replace('"', '&quot;', $form->createMarkup());
            $text = SFUtils::printRedirectForm($title, $full_text, "", $save_page, $preview_page, false, false, false, null, null);
            $wgOut->addHTML($text);
            return ('0'); //suppress the the query page
        }
    }

    $text = '	<form action="" method="post">' . "\n";
    // set 'title' field, in case there's no URL niceness
    $mw_namespace_labels = $wgContLang->getNamespaces();
    $special_namespace = $mw_namespace_labels[NS_SPECIAL];
    $text .= '    <input type="hidden" name="title" value="' . $special_namespace . ':SFF_EditForm">' . "\n";
    $text .= '
  <p><div id="acForm">' . wfMsg('sf_createform_nameinput') .
 '  <input id="form_name" type="text" size="25" name="form_name" value="' . $form_name . '">
    <div id="cntrForm"> </div>
  </div>';

    if (! empty($form_name_error_str))
    $text .= ' <font color="red">' . $form_name_error_str . '</font>';
    $text .= "</p>\n";

    $text .= $form->creationHTML();

    // Buttons maintain form and add template:
    if (count($form_templates) == 0) {
        $text .=<<<END
    <p><input type="submit" id="maintain_form" name="maintain_form" value="$edit_form_text"></p>
END;
    }
    // Make link for edit template:
    $sk = $wgUser->getSkin();
    $et = SpecialPage::getPage('SFF_EditTemplate');
    $edit_template_link = $sk->makeKnownLinkObj($et->getTitle(), $et->getDescription());
    if  ( ($form->form_name != "") || (count($form_templates) > 0) ) {
        $text .= '	<p>' . wfMsg('sf_createform_addtemplate') . "\n";
        $text .= '	<select name="new_template">' . "\n";

        foreach ($all_templates as $template) {
            $text .= "	<option value=\"$template\">$template</option>\n";
        }

        $text .= "	</select>\n";

        // if a template has already been added, show a dropdown letting the
        // user choose where in the list to add a new dropdown
        if (count($form_templates) > 0) {
            $before_template_msg = wfMsg('sf_createform_beforetemplate');
            $at_end_msg = wfMsg('sf_createform_atend');
            $text .=<<<END
            $before_template_msg
    <select name="before_template">

END;
            foreach ($form_templates as $i => $ft) {
                $text .= "	<option value=\"$i\">{$ft->template_name}</option>\n";
            }
            $final_index = count($form_templates);
            $text .=<<<END
    <option value="$final_index" selected="selected">$at_end_msg</option>
    </select>

END;
        }
        $add_button_text = wfMsg('sf_createform_add');
        $text .=<<<END
    <input type="submit" name="add_template" value="$add_button_text"></p>
END;

    }


    // disable 'save' and 'preview' buttons if user has not yet added any
    // templates
    $disabled_text = (count($form_templates) == 0) ? "disabled" : "";
    $save_button_text = wfMsg('savearticle');
    $preview_button_text = wfMsg('preview');
    $text .=<<<END

    <br />
    <div class="editButtons">
    <p>
    <input type="submit" id="wpSave" name="wpSave" value="$save_button_text" $disabled_text />
    <input type="submit" id="wpPreview" name="wpPreview" value="$preview_button_text" $disabled_text />
    </p>
    </div>

END;
    // explanatory message if buttons are disabled because no templates
    // have been added
    if (count($form_templates) == 0) {
        $text .= "	<p>(" . wfMsg('sf_createtemplate_addtemplatebeforesave') . ")</p>";
    }
    $text .=<<<END
    </form>
    <hr /><br />

    <p>$edit_template_link.</p>

END;

// Provide autocompletion for field(s):
    $text .= SFFUtils::getCSSText();
    $text .='<script type="text/javascript">/* <![CDATA[ */ ';
    $text .= SFFUtils::buildArrayFromNameSpace(SF_NS_FORM,"localFormList");
    $text .='var myDS1 = new YAHOO.widget.DS_JSArray (localFormList);
  myAutoComp1 = new YAHOO.widget.AutoComplete("form_name", "cntrForm", myDS1);
/* ]]> */ </script>';

    $wgOut->addLink( array(
        'rel' => 'stylesheet',
        'type' => 'text/css',
        'media' => "screen, projection",
        'href' => $sfgScriptPath . "/skins/SF_main.css"
        ));
    $wgOut->addHTML($text);
    return ('1');
}

# Function is copied from the source SF_FormPrinter, function formHTML
# and is modified for usage in the Special page SFF_EditForm
function doEditExistingForm($form_name, $form_templates, $db) {

    global $wgOut;

    # test before process if form realy exists else add form:
    $form_name_error_str = '';
    $form_name_db = str_replace(' ', '_', $form_name);
    $sql = "SELECT * FROM {$db->tableName( 'page' )}
                WHERE page_namespace=" . SF_NS_FORM . " AND page_title='$form_name_db'";
    $res = $db->query( $sql );
    if ($db->numRows( $res ) > 0) {
        $form_exists=true;

        # page-contents from existing form to array
        $form_title = Title::newFromText($form_name_db, SF_NS_FORM);
        $form_article = new Article($form_title);
        $form_def = $form_article->getContent();

        # change noinclude and includeonly sections
        $form_def = StringUtils::delimiterReplace('<noinclude>', '</noinclude>', '', $form_def);
        //$form_def = strtr($form_def, array('<noinclude>' => '{{{noinclude}}}', '</noinclude>' => '{{{/noinclude}}}'));
        $form_def = strtr($form_def, array('<includeonly>' => '', '</includeonly>' => ''));
        //$form_def = strtr($form_def, array('<includeonly>' => '{{{includeonly}}}', '</includeonly>' => '/includeonly}}}'));

        # turn form definition file into an array of sections, one for each
        # template definition (plus the first section)
        $form_def_sections = array();
        $start_position = 0;
        $section_start = 0;
        $free_text_was_included = false;
        $free_text_preload_page = null;
        $all_values_for_template = array();
        // unencode and HTML-encoded representations of curly brackets and
        // pipes - this is a hack to allow for forms to include templates
        // that themselves contain form elements - the escaping is needed
        // to make sure that those elements don't get parsed too early
        $form_def = str_replace(array('&#123;', '&#124;', '&#125;'), array('{', '|', '}'), $form_def);
        // and another hack - replace the 'free text' standard input with
        // a field declaration to get it to be handled as a field
        $form_def = str_replace('standard input|free text', 'field|<freetext>', $form_def);
        while ($brackets_loc = strpos($form_def, "{{{", $start_position)) {
            $brackets_end_loc = strpos($form_def, "}}}", $brackets_loc);
            $bracketed_string = substr($form_def, $brackets_loc + 3, $brackets_end_loc - ($brackets_loc + 3));
            $tag_components = explode('|', $bracketed_string);
            $tag_title = trim($tag_components[0]);
            if ($tag_title == 'for template' || $tag_title == 'end template') {
                // create a section for everything up to here
                $section = substr($form_def, $section_start, $brackets_loc - $section_start);
                $form_def_sections[] = $section;
                $section_start = $brackets_loc;
            }
            $start_position = $brackets_loc + 1;
        } // end while
        $form_def_sections[] = trim(substr($form_def, $section_start));

        // cycle through form definition file (and possibly an existing article
        // as well), finding template and field declarations and replacing them
        // with form elements, either blank or pre-populated, as appropriate
        $all_fields = array();
        $data_text = "";
        $template_name = "";
        $template_label = "";
        $allow_multiple = false;
        $instance_num = 0;
        $all_instances_printed = false;
        $strict_parsing = false;
        // initialize list of choosers (dropdowns with available templates)
        $choosers = array();
        for ($section_num = 0; $section_num < count($form_def_sections); $section_num++) {
            //$tif = new SFFTemplateInForm();
            $start_position = 0;
            $template_text = "";
            // the append is there to ensure that the original array doesn't get
            // modified; is it necessary?
            $section = ' ' . $form_def_sections[$section_num];
            while ( $brackets_loc = strpos($section, '{{{', $start_position) ) {
                $brackets_end_loc = strpos($section, "}}}", $brackets_loc);
                $bracketed_string = substr($section, $brackets_loc + 3, $brackets_end_loc - ($brackets_loc + 3));
                $tag_components = explode('|', $bracketed_string);
                $tag_title = trim($tag_components[0]);
                // =====================================================
                // for template processing
                // =====================================================
                if ($tag_title == 'for template') {
                    $old_template_name = $template_name;
                    $template_name = trim($tag_components[1]);
                    //$tif->template_name = $template_name;
                    $query_template_name = str_replace(' ', '_', $template_name);
                    // also replace periods with underlines, since that's what
                    // POST does to strings anyway
                    $query_template_name = str_replace('.', '_', $query_template_name);
                    $chooser_name = false;
                    $chooser_caption = false;
                    // cycle through the other components
                    for ($i = 2; $i < count($tag_components); $i++) {
                        $component = $tag_components[$i];
                        if ($component == 'multiple') $allow_multiple = true;
                        if ($component == 'strict') $strict_parsing = true;
                        $sub_components = explode('=', $component);
                        if (count($sub_components) == 2) {
                            if ($sub_components[0] == 'label') {
                                $template_label = $sub_components[1];
                            } elseif ($sub_components[0] == 'chooser') {
                                $allow_multiple = true;
                                $chooser_name = $sub_components[1];
                            } elseif ($sub_components[0] == 'chooser caption') {
                                $chooser_caption = $sub_components[1];
                            }
                        }
                    }

                    # Create new template
                    $form_template = SFFTemplateInForm::create($template_name, $template_label, $allow_multiple);
                    # save all standard fields and clear them
                    $all_template_fields = $form_template->getAllFields();

                    //$new_template_loc = 0;

                    # hack - array_splice() doesn't work for objects, so we have to
                    # first insert a stub element into the array, then replace that
                    # with the actual object
                    $form_templates[] = $form_template;

                    // remove template tag
                    $section = substr_replace($section, '', $brackets_loc, $brackets_end_loc + 3 - $brackets_loc);
                    $brackets_end_loc = 0;
                }
                // =====================================================
                // end template processing
                // =====================================================
                elseif ($tag_title == 'end template') {
                    // remove this tag, reset some variables, and close off form HTML tag
                    $section = substr_replace($section, '', $brackets_loc, $brackets_end_loc + 3 - $brackets_loc);
                    $brackets_end_loc = $brackets_loc;
                    $template_name = null;
                    if (isset($template_label)) {
                        //$form_text .= "</fieldset>\n";
                        unset ($template_label);
                    }
                    $allow_multiple = false;
                    $all_instances_printed = false;
                    $instance_num = 0;
                }
                // =====================================================
                // field processing
                // =====================================================
                elseif ($tag_title == 'field') {
                    $field_name = trim($tag_components[1]);
                    $is_mandatory = false;
                    $is_hidden = false;
                    $is_restricted = false;
                    $is_uploadable = false;
                    $is_list = false;
                    $input_type = null;
                    $field_args = array();
                    $default_value = "";
                    $possible_values = null;
                    $preload_page = null;
                    for ($i = 2; $i < count($tag_components); $i++) {
                        $component = trim($tag_components[$i]);
                        if ($component == 'mandatory') {
                            $is_mandatory = true;
                        } elseif ($component == 'hidden') {
                            $is_hidden = true;
                        } elseif ($component == 'restricted') {
                            $is_restricted = true;
                        } elseif ($component == 'uploadable') {
                            $field_args['is_uploadable'] = true;
                        } elseif ($component == 'list') {
                            $is_list = true;
                        } elseif ($component == 'autocomplete') {
                            $field_args['autocomplete'] = true;
                        } elseif ($component == 'no autocomplete') {
                            $field_args['no autocomplete'] = true;
                        } elseif ($component == 'remote autocompletion') {
                            $field_args['remote autocompletion'] = true;
                        } else {
                            $sub_components = explode('=', $component);
                            if (count($sub_components) == 2) {
                                if ($sub_components[0] == 'input type') {
                                    $input_type = $sub_components[1];
                                } elseif ($sub_components[0] == 'default') {
                                    $default_value = $sub_components[1];
                                } elseif ($sub_components[0] == 'preload') {
                                    // free text field has special handling
                                    if ($field_name == 'free text' || $field_name = '<freetext>') {
                                        $free_text_preload_page = $sub_components[1];
                                    } else {
                                        // this variable is not used
                                        $preload_page = $sub_components[1];
                                    }
                                } elseif ($sub_components[0] == 'values') {
                                    $possible_values = explode(',', $sub_components[1]);
                                } elseif ($sub_components[0] == 'values from category') {
                                    $possible_values = SFUtils::getAllPagesForCategory($sub_components[1], 10);
                                } elseif ($sub_components[0] == 'values from concept') {
                                    $possible_values = SFUtils::getAllPagesForConcept($sub_components[1]);
                                } else {
                                    $field_args[$sub_components[0]] = $sub_components[1];
                                }
                                // for backwards compatibility
                                if ($sub_components[0] == 'autocomplete on' && $sub_components[1] == null) {
                                    $field_args['no autocomplete'] = true;
                                }
                            }
                        }


                    } //for loop in field

                    // read label of the next field: pattern "! <field>:" or "''' <field>:"
                    $search_label = '';
                    if ($startpos_label = strpos($section, '!', $start_position)) {
                        if ($endpos_label = strpos($section, ':', $startpos_label)) {
                            $search_label = substr($section, $startpos_label + 2, $endpos_label - ($startpos_label + 2));
                        }
                    }
                    else {
                        if ($startpos_label = strpos($section, "'''", $start_position)) {
                            if ($endpos_label = strpos($section, ':', $startpos_label)) {
                                $search_label = substr($section, $startpos_label + 3, $endpos_label - ($startpos_label + 3));
                            }
                        }
                    }

                    # special fields as "free text" not processed here; not a template field!!
                    if ($field_name != '' && $field_name != 'free text' && $field_name != '<freetext>') {
                        $is_disabled = false;
                        $strict_parsing = false;
                        $input_name = $search_label;
                        $label = $search_label;
                        //$field_count = count($form_template->fields);
                        //Overwrite the default template field values with the exising values:
                        foreach ($form_template->fields as $form_field2) {
                            if ($field_name == $form_field2->template_field->field_name ) {
                                //Instance SFTemplateField:
                                $form_field2->template_field->label = $label;
                                $form_field2->template_field->input_type = $input_type;
                                $form_field2->template_field->field_name = $field_name;
                                $form_field2->template_field->possible_values = $possible_values;
                                //Instance SFFFormTemplateField:
                                $form_field2->is_mandatory = $is_mandatory;
                                $form_field2->is_restricted = $is_restricted;
                                $form_field2->is_hidden = $is_hidden;
                                $form_field2->is_uploadable = $is_uploadable;
                                $form_field2->possible_values = $possible_values;
                                $form_field2->input_type = $input_type;
                                $form_field2->field_args = $field_args;
                                $form_field2->input_name = $input_name;
                            }
                            else {
                                $form_field2->template_field->input_type = '';
                            }
                        }
                    }
                }
                // =====================================================
                // standard input processing
                // =====================================================
                elseif ($tag_title == 'standard input') {
                    $brackets_end_loc = strpos($section, "}}}", $brackets_loc);

                }
                //
                // =====================================================
                // page info processing
                // =====================================================
                elseif ($tag_title == 'info') {
                    $brackets_end_loc = strpos($section, "}}}", $brackets_loc);

                }
                //
                //
                // =====================================================
                // default outer level processing
                // =====================================================
                else { // tag is not one of the three allowed values
                    // ignore tag
                } // end if
                # set for next while loop
                $start_position = $brackets_end_loc;
            } //while loop
        } //for loop
    } else {
        $form_exists=false;
    }
    return $form_templates;
}